from abc import ABC, abstractmethod
from typing import Any

from chosco.annotation.domain.aggregate_roots.media_item import MediaItem


class MediaRepository(ABC):
    @abstractmethod
    def retrieve(self, media_item: MediaItem, width: int = None) -> Any:
        raise NotImplementedError



from IPython.core.display import Image

from chosco.annotation.domain.aggregate_roots.media_item import MediaItem
from chosco.media.repository.domain.media_repository import MediaRepository


class HardcodedWebMediaRepository(MediaRepository):
    def retrieve(self, media_item: MediaItem, width: int = None) -> Image:
        try:
            return Image(media_item.media_id, unconfined=True, width=width)
        except:  # noqa
            raise IndexError(
                f"media_id {media_item.media_id} cannot be loaded on HardcodedWebMediaRepository"
            )



import json

from chosco.annotation.domain.annotation import Annotation
from chosco.annotation.domain.annotation_repository import AnnotationRepository


class InMemoryAnnotationRepository(AnnotationRepository):
    def __init__(self):
        self.items = {}

    def save(self, annotation: Annotation):
        if annotation.username not in self.items:
            self.items[annotation.username] = {}
        self.items[annotation.username][annotation.id] = annotation.metadata

    def retrieve(self, annotation_id: str, username: str) -> Annotation:
        if username not in self.items:
            raise ValueError(f"username {username} not exist in [{self.items.keys()}]")

        username_items = self.items.get(username)

        if annotation_id not in username_items:
            raise ValueError(
                f"annotation_id {username} not exist in [{username_items}]"
            )

        return username_items.get(annotation_id)

    def show(self):
        print(json.dumps(self.items, indent=4))



import json


class Annotation:
    def __init__(self, id: str, username: str, metadata: dict):
        self.id = id
        self.username = username
        self.metadata = metadata

    def __repr__(self):
        kdict = {"id": self.id, "username": self.username, "metadata": self.metadata}
        return json.dumps(kdict, indent=4)
